@@ -1,7 +1,7 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2018 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2020 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -54,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x04130a
-#define SIP_VERSION_STR     "4.19.10"
+#define SIP_VERSION         0x041316
+#define SIP_VERSION_STR     "4.19.22"
 
 
 /*
@@ -68,6 +68,17 @@ extern "C" {
  *
  * History:
  *
+ * 12.7 Added sip_api_visit_wrappers() to the public API.
+ *      Added sip_api_register_exit_notifier() to the public API.
+ *      sip_api_is_owned_by_python() is now part of the public API.
+ *
+ * 12.6 Added sip_api_long_as_size_t() to the public API.
+ *      Added the '=' format character to sip_api_build_result().
+ *      Added the '=' format character to sip_api_parse_result_ex().
+ *
+ * 12.5 Replaced the sipConvertFromSliceObject() macro with
+ *      sip_api_convert_from_slice_object() in the public API.
+ *
  * 12.4 Added sip_api_instance_destroyed_ex() to the private API.
  *
  * 12.3 Added SIP_TYPE_SCOPED_ENUM to the sipTypeDef flags.
@@ -265,7 +276,7 @@ extern "C" {
  * 0.0  Original version.
  */
 #define SIP_API_MAJOR_NR    12
-#define SIP_API_MINOR_NR    4
+#define SIP_API_MINOR_NR    7
 
 
 /*
@@ -278,6 +289,41 @@ extern "C" {
 typedef unsigned int uint;
 
 
+/* Some C++ compatibility stuff. */
+#if defined(__cplusplus)
+
+/*
+ * Cast a PyCFunctionWithKeywords to a PyCFunction in such a way that it
+ * suppresses the GCC -Wcast-function-type warning.
+ */
+#define SIP_MLMETH_CAST(m)  reinterpret_cast<PyCFunction>(reinterpret_cast<void (*)(void)>(m))
+
+#if __cplusplus >= 201103L || defined(_MSVC_LANG)
+
+/* C++11 and later. */
+#define SIP_NULLPTR     nullptr
+#define SIP_OVERRIDE    override
+
+#else
+
+/* Earlier versions of C++. */
+#define SIP_NULLPTR     NULL
+#define SIP_OVERRIDE
+
+#endif
+
+#else
+
+/* Cast a PyCFunctionWithKeywords to a PyCFunction. */
+#define SIP_MLMETH_CAST(m)  ((PyCFunction)(m))
+
+/* C. */
+#define SIP_NULLPTR     NULL
+#define SIP_OVERRIDE
+
+#endif
+
+
 /* Some Python compatibility stuff. */
 #if PY_VERSION_HEX >= 0x02050000
 
@@ -495,6 +541,7 @@ typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *, PyObject *);
 typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
 typedef void *(*sipProxyResolverFunc)(void *);
 typedef int (*sipNewUserTypeFunc)(sipWrapperType *);
+typedef void (*sipWrapperVisitorFunc)(sipSimpleWrapper *, void *);
 
 
 #if !defined(Py_LIMITED_API) || PY_VERSION_HEX < 0x03020000
@@ -1829,7 +1876,15 @@ typedef struct _sipAPIDef {
     int (*api_init_mixin)(PyObject *self, PyObject *args, PyObject *kwds,
             const sipClassTypeDef *ctd);
     PyObject *(*api_get_reference)(PyObject *self, int key);
+
+    /*
+     * The following are part of the public API.
+     */
     int (*api_is_owned_by_python)(sipSimpleWrapper *);
+
+    /*
+     * The following are not part of the public API.
+     */
     int (*api_is_derived_class)(sipSimpleWrapper *);
 
     /*
@@ -1889,6 +1944,16 @@ typedef struct _sipAPIDef {
      * The following are not part of the public API.
      */
     void (*api_instance_destroyed_ex)(sipSimpleWrapper **sipSelfp);
+
+    /*
+     * The following are part of the public API.
+     */
+    int (*api_convert_from_slice_object)(PyObject *slice, SIP_SSIZE_T length,
+            SIP_SSIZE_T *start, SIP_SSIZE_T *stop, SIP_SSIZE_T *step,
+            SIP_SSIZE_T *slicelength);
+    size_t (*api_long_as_size_t)(PyObject *o);
+    void (*api_visit_wrappers)(sipWrapperVisitorFunc visitor, void *closure);
+    int (*api_register_exit_notifier)(PyMethodDef *md);
 } sipAPIDef;
 
 
@@ -2007,18 +2072,6 @@ typedef struct _sipQtAPI {
 #define sipTypeName(td)     sipNameFromPool((td)->td_module, (td)->td_cname)
 #define sipTypePluginData(td)   ((td)->td_plugin_data)
 
-#if PY_VERSION_HEX >= 0x03070000
-#define sipConvertFromSliceObject(o, len, start, stop, step, slen) (        \
-    PySlice_Unpack((o), (start), (stop), (step)) < 0 ?                      \
-    ((*(slen) = 0), -1) :                                                   \
-    ((*(slen) = PySlice_AdjustIndices((len), (start), (stop), *(step))), 0))
-#elif PY_VERSION_HEX >= 0x03020000
-#define sipConvertFromSliceObject   PySlice_GetIndicesEx
-#else
-#define sipConvertFromSliceObject(o, len, start, stop, step, slen)          \
-        PySlice_GetIndicesEx((PySliceObject *)(o), (len), (start), (stop),  \
-                (step), (slen))
-#endif
 
 /*
  * Note that this was never actually documented as being part of the public
